

/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2020  SEGGER Microcontroller GmbH                *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V6.10 - Graphical user interface for embedded applications **
emWin is protected by international copyright laws.   Knowledge of the
source code may not be used to write a similar product.  This file may
only  be used  in accordance  with  a license  and should  not be  re-
distributed in any way. We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : LISTBOX_AutoScrollText.c
Purpose     : Sample that demonstrates how to implement automatic text
              text scrolling in a LISTBOX widget.
Requirements: WindowManager - (x)
              MemoryDevices - ( )
              AntiAliasing  - ( )
              VNC-Server    - ( )
              PNG-Library   - ( )
              TrueTypeFonts - ( )
---------------------------END-OF-HEADER------------------------------
*/

#include "DIALOG.h"
#include <stdio.h>
#include <stdlib.h>

/*********************************************************************
*
*       Types
*
**********************************************************************
*/
typedef struct {
  WM_HWIN hWin;
  int     ItemIndex;
  int     xPosEnd;
  int     xOffset;
} ANIM_DATA;

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define SPACING       5
#define SCROLL_PERIOD 3000
#define WAIT_PERIOD   1500

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static ANIM_DATA _aAnimData[11];
static int       _aNumData;

static const char * _acContent[] = {
  "English",
  "Deutsch",
  "Bahasa Indonesia",
  "Espanol",
  "Francais",
  "Italiano",
  "Dansk",
  "Mandarin Chinese",
  "Hrvatski",
  "Portugues",
  "Suomi",
  NULL
};

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/
/*********************************************************************
*
*       _AnimMoveText
*/
static void _AnimMoveText(GUI_ANIM_INFO * pInfo, void * pVoid) {
  ANIM_DATA  * pData;

  pData = (ANIM_DATA *)pVoid;
  pData->xOffset = -((pData->xPosEnd * pInfo->Pos) / GUI_ANIM_RANGE);
  //
  // Redraw window
  //
  WM_InvalidateWindow(pData->hWin);
}

/*********************************************************************
*
*       _AnimWait
*/
static void _AnimWait(GUI_ANIM_INFO * pInfo, void * pVoid) {
  //
  // Do nothing...
  //
}

/*********************************************************************
*
*       _OwnerDrawListbox
*/
static int _OwnerDrawListbox(const WIDGET_ITEM_DRAW_INFO * pDrawItemInfo) {
  int      Sel;
  char     acBuffer[32];
  int      i;
  int      xOffset;
  GUI_RECT ClipRect;

  switch (pDrawItemInfo->Cmd) {
  case WIDGET_ITEM_DRAW:
    if (pDrawItemInfo->ItemIndex > -1) { // -1 is an invalid selection
      //
      // Draw background
      //
      Sel = LISTBOX_GetSel(pDrawItemInfo->hWin);
      if (pDrawItemInfo->ItemIndex == Sel) {
        GUI_SetColor(GUI_LIGHTBLUE);
      } else {
        GUI_SetColor(GUI_WHITE);
      }
      GUI_FillRect(pDrawItemInfo->x0, pDrawItemInfo->y0, pDrawItemInfo->x1, pDrawItemInfo->y1);
      //
      // Draw item text
      //
      LISTBOX_GetItemText(pDrawItemInfo->hWin, pDrawItemInfo->ItemIndex, acBuffer, sizeof(acBuffer));
      GUI_SetTextMode(GUI_TM_TRANS);
      if (pDrawItemInfo->ItemIndex == Sel) {
        GUI_SetColor(GUI_WHITE);
      } else {
        GUI_SetColor(GUI_BLACK);
      }
      //
      // Check if for current item a text animation is running
      //
      for (i = 0; i < _aNumData; i++) {
        if (_aAnimData[i].ItemIndex ==  pDrawItemInfo->ItemIndex) {
          xOffset = _aAnimData[i].xOffset;
          break;
        } else {
          xOffset = 0;
        }
      }
      //
      // Set up clipping rectangle
      //
      ClipRect.x0 = pDrawItemInfo->x0 + SPACING + 1;
      ClipRect.y0 = pDrawItemInfo->y0;
      ClipRect.x1 = pDrawItemInfo->x1;
      ClipRect.y1 = pDrawItemInfo->y1;
      WM_SetUserClipRect(&ClipRect);
      //
      // Draw text
      //
      GUI_DispStringAt(acBuffer, pDrawItemInfo->x0 + SPACING + xOffset, pDrawItemInfo->y0);
      //
      // Clear clipping rectangle
      //
      WM_SetUserClipRect(NULL);
    }
    return 0;
  default:
    return LISTBOX_OwnerDraw(pDrawItemInfo);
  }
}

/*********************************************************************
*
*       _cbListbox
*/
static void _cbListbox(WM_MESSAGE * pMsg) {
  int              i;
  int              NumItems;
  char             acBuffer[64];
  const GUI_FONT * pOldFont;
  int              xSizeListbox;
  int              xSizeText;
  ANIM_DATA      * pData;
  GUI_ANIM_HANDLE  hAnim;

  switch (pMsg->MsgId) {
  case WM_SET_CALLBACK:
    pOldFont     = GUI_SetFont(LISTBOX_GetFont(pMsg->hWin));
    xSizeListbox = WM_GetXSize(pMsg->hWin) - (SPACING * 2);
    //
    // Check string length and initialize animations
    //
    NumItems = LISTBOX_GetNumItems(pMsg->hWin);
    for (i = 0; i < NumItems; i++) {
      LISTBOX_GetItemText(pMsg->hWin, i, acBuffer, sizeof(acBuffer));
      xSizeText = GUI_GetStringDistX(acBuffer);
      pData     = &_aAnimData[_aNumData];
      //
      // Check if we need an animation for the current item
      //
      if (xSizeText > xSizeListbox) {
        pData->hWin      = pMsg->hWin;
        pData->ItemIndex = i;
        pData->xPosEnd   = (xSizeText - xSizeListbox); // Length of overlapping text is the distance we want to scroll
        pData->xOffset   = 0;
        //
        // Create and start animations
        //
        hAnim = GUI_ANIM_Create(SCROLL_PERIOD + WAIT_PERIOD, 25, NULL, NULL);
        GUI_ANIM_AddItem(hAnim, 0, SCROLL_PERIOD, ANIM_LINEAR, (void *)pData, _AnimMoveText);              // Add item for moving text
        GUI_ANIM_AddItem(hAnim, SCROLL_PERIOD, SCROLL_PERIOD + WAIT_PERIOD, ANIM_LINEAR, NULL, _AnimWait); // Add item for pause after text move
        GUI_ANIM_StartEx(hAnim, -1, NULL);                                                                 // Loop animation endlessly
        _aNumData++;
      }
    }
    GUI_SetFont(pOldFont);
    break;
  default:
    LISTBOX_Callback(pMsg);
  }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       MainTask
*/
void MainTask(void) {
  LISTBOX_Handle hListbox;

  //
  // Init GUI
  //
  GUI_Init();
  //
  // Enable multi-buffering for smooth animation
  //
  WM_MULTIBUF_Enable(1);
  //
  // Create listbox widget
  //
  hListbox = LISTBOX_CreateEx(10, 10, 120, 185, WM_HBKWIN, WM_CF_SHOW, 0, GUI_ID_LISTBOX0, _acContent);
  LISTBOX_SetFont(hListbox, &GUI_Font20_ASCII);
  //
  // Set OwnerDraw function
  //
  LISTBOX_SetOwnerDraw(hListbox, _OwnerDrawListbox);
  //
  // Set LISTBOX callback
  //
  WM_SetCallback(hListbox, _cbListbox);

  while (1) {
    GUI_Delay(100);
  }
}

/*************************** End of file ****************************/

